@@ -10,11 +10,13 @@ import {
     PropertyMapPickInfo,
     ValueDecoder,
 } from "../utils/propertyMapTools";
-import { getModelMatrix } from "../utils/layerTools";
+import { getModelMatrix, colorMapFunctionType } from "../utils/layerTools";
 import { layersDefaultProps } from "../layersDefaultProps";
-import fsColormap from "!!raw-loader!./colormap.fs.glsl";
-import { DeckGLLayerContext } from "../../components/DeckGLWrapper";
-import { colorTablesArray, rgbValues } from "@emerson-eps/color-tables/";
+import fsColormap from "./colormap.fs.glsl";
+import { DeckGLLayerContext } from "../../components/Map";
+import { colorTablesArray } from "@emerson-eps/color-tables/";
+import { getRgbData } from "@emerson-eps/color-tables";
+import { ContinuousLegendDataType } from "../../components/ColorLegend";
 
 const DEFAULT_TEXTURE_PARAMETERS = {
     [GL.TEXTURE_MIN_FILTER]: GL.LINEAR_MIPMAP_LINEAR,
@@ -23,12 +25,20 @@ const DEFAULT_TEXTURE_PARAMETERS = {
     [GL.TEXTURE_WRAP_T]: GL.CLAMP_TO_EDGE,
 };
 
-function getImageData(colorMapName: string, colorTables: colorTablesArray) {
+function getImageData(
+    colorMapName: string,
+    colorTables: colorTablesArray,
+    colorMapFunction: colorMapFunctionType | undefined
+) {
+    const isColorMapFunctionDefined = typeof colorMapFunction !== "undefined";
+
     const data = new Uint8Array(256 * 3);
 
     for (let i = 0; i < 256; i++) {
         const value = i / 255.0;
-        const rgb = rgbValues(value, colorMapName, colorTables);
+        const rgb = isColorMapFunctionDefined
+            ? (colorMapFunction as colorMapFunctionType)(i / 255)
+            : getRgbData(value, colorMapName, colorTables);
         let color: number[] = [];
         if (rgb != undefined) {
             if (Array.isArray(rgb)) {
@@ -68,6 +78,11 @@ export interface ColormapLayerProps<D> extends BitmapLayerProps<D> {
     // Name of color map.
     colorMapName: string;
 
+    // Optional function property.
+    // If defined this function will override the color map.
+    // Takes a value in the range [0,1] and returns a color.
+    colorMapFunction?: colorMapFunctionType;
+
     // Min and max property values.
     valueRange: [number, number];
 
@@ -127,7 +142,8 @@ export default class ColormapLayer extends BitmapLayer<
                     data: getImageData(
                         this.props.colorMapName,
                         (this.context as DeckGLLayerContext).userData
-                            .colorTables
+                            .colorTables,
+                        this.props.colorMapFunction
                     ),
                     parameters: DEFAULT_TEXTURE_PARAMETERS,
                 }),
@@ -172,6 +188,24 @@ export default class ColormapLayer extends BitmapLayer<
             propertyValue: val,
         };
     }
+
+    getLegendData(): ContinuousLegendDataType {
+        const valueRangeMin = this.props.valueRange[0] ?? 0.0;
+        const valueRangeMax = this.props.valueRange[1] ?? 1.0;
+
+        // If specified color map will extend from colorMapRangeMin to colorMapRangeMax.
+        // Otherwise it will extend from valueRangeMin to valueRangeMax.
+        const min = this.props.colorMapRange?.[0] ?? valueRangeMin;
+        const max = this.props.colorMapRange?.[1] ?? valueRangeMax;
+
+        return {
+            discrete: false,
+            valueRange: [min, max],
+            colorName: this.props.colorMapName,
+            title: "PropertyMapLayer",
+            colorMapFunction: this.props.colorMapFunction,
+        };
+    }
 }
 
 ColormapLayer.layerName = "ColormapLayer";
