@@ -1,5 +1,4 @@
 import { COORDINATE_SYSTEM, RGBAColor } from "@deck.gl/core";
-import { UpdateStateInfo } from "@deck.gl/core/lib/layer";
 import { ExtendedLayerProps } from "../utils/layerTools";
 import {
     DrawLineStringMode,
@@ -8,6 +7,7 @@ import {
     EditAction,
     Feature,
     FeatureCollection,
+    GeoJsonEditMode,
     ImmutableFeatureCollection,
     ModeProps,
     ModifyMode,
@@ -16,7 +16,9 @@ import {
 } from "@nebula.gl/edit-modes";
 import { EditableGeoJsonLayer } from "@nebula.gl/layers";
 import { CompositeLayer, PickInfo } from "deck.gl";
-import { patchLayerProps } from "../utils/layerTools";
+import { layersDefaultProps } from "../layersDefaultProps";
+import { DeckGLLayerContext } from "../../components/Map";
+import { area, length } from "../../utils/measurement";
 
 // Custom drawing mode that deletes the selected GeoJson feature when releasing the Delete key.
 class CustomModifyMode extends ModifyMode {
@@ -41,100 +43,152 @@ class CustomModifyMode extends ModifyMode {
     }
 }
 
+function deleteEscapeKeyHandler(
+    drawMode: GeoJsonEditMode,
+    event: KeyboardEvent,
+    props: ModeProps<FeatureCollection>
+) {
+    if (event.key === "Escape") drawMode.getClickSequence().pop();
+    else if (event.key === "Delete") drawMode.resetClickSequence();
+    else return;
+
+    // used to set state so layer can be rerendered
+    const updatedData = new ImmutableFeatureCollection(props.data).getObject();
+    if (updatedData) {
+        props.onEdit({
+            updatedData,
+            editType: "undoDrawing",
+            editContext: {
+                featureIndexes: props.selectedIndexes,
+            },
+        });
+    }
+}
+
+class CustomDrawLineStringMode extends DrawLineStringMode {
+    handleKeyUp(event: KeyboardEvent, props: ModeProps<FeatureCollection>) {
+        super.handleKeyUp(event, props);
+        deleteEscapeKeyHandler(this, event, props);
+    }
+}
+
+class CustomDrawPolygonMode extends DrawPolygonMode {
+    handleKeyUp(event: KeyboardEvent, props: ModeProps<FeatureCollection>) {
+        super.handleKeyUp(event, props);
+        deleteEscapeKeyHandler(this, event, props);
+    }
+}
+
 // Mapping of mode name to mode class
 const MODE_MAP = {
     view: ViewMode,
     modify: CustomModifyMode,
     transform: TransformMode,
     drawPoint: DrawPointMode,
-    drawLineString: DrawLineStringMode,
-    drawPolygon: DrawPolygonMode,
+    drawLineString: CustomDrawLineStringMode,
+    drawPolygon: CustomDrawPolygonMode,
 };
 
 const UNSELECTED_LINE_COLOR: RGBAColor = [0x50, 0x50, 0x50, 0xcc];
 const SELECTED_LINE_COLOR: RGBAColor = [0x0, 0x0, 0x0, 0xff];
 
-const defaultProps = {
-    name: "Drawing",
-    pickable: true,
-    mode: "drawLineString",
-
-    // Props mainly used to make the information available to the Map parent comp.
-    selectedFeatureIndexes: [],
-    data: {
-        type: "FeatureCollection",
-        features: [],
-    },
-};
-
 export interface DrawingLayerProps<D> extends ExtendedLayerProps<D> {
     mode: string; // One of modes in MODE_MAP
     selectedFeatureIndexes: number[];
 }
 
+export interface DrawingPickInfo extends PickInfo<unknown> {
+    measurement?: number;
+}
+
 // Composite layer that contains an EditableGeoJsonLayer from nebula.gl
 // See https://nebula.gl/docs/api-reference/layers/editable-geojson-layer
 export default class DrawingLayer extends CompositeLayer<
     FeatureCollection,
     DrawingLayerProps<FeatureCollection>
 > {
-    updateState(
-        info: UpdateStateInfo<DrawingLayerProps<FeatureCollection>>
-    ): void {
-        super.updateState(info);
+    initializeState(params?: PickInfo<FeatureCollection>): void {
+        super.initializeState(params);
 
-        if (info.changeFlags.dataChanged) {
-            this.setState({
-                data: this.props.data,
-            });
-        }
+        this.setState({
+            data: this.props.data,
+            selectedFeatureIndexes: this.props.selectedFeatureIndexes,
+        });
     }
 
     // Select features when clicking on them if in view or modify modes.
-    // The selection is sent to the map component parent as a patch.
+    // The selection is used to set current selected drawing, and
+    // is sent to the map component parent via setEditedData.
     onClick(info: PickInfo<FeatureCollection>): boolean {
         if (this.props.mode === "view" || this.props.mode === "modify") {
-            const featureIndex = this.state.data.features.indexOf(info.object);
-            if (featureIndex >= 0) {
-                patchLayerProps<FeatureCollection>(this, {
-                    ...this.props,
-                    selectedFeatureIndexes: [info.index],
-                } as DrawingLayerProps<FeatureCollection>);
-                return true;
-            }
+            this.setState({
+                selectedFeatureIndexes: [info.index],
+            });
+            (this.context as DeckGLLayerContext).userData.setEditedData({
+                selectedFeatureIndexes: [info.index],
+            });
+            return true;
         }
 
         return false;
     }
 
+    // For now, use `any` for the picking types because this function should
+    // recieve PickInfo<FeatureCollection>, but it recieves PickInfo<Feature>.
+    //eslint-disable-next-line
+    getPickingInfo({ info }: { info: any }): any {
+        if (!info.object) return info as DrawingPickInfo;
+        const feature = info.object;
+        let measurement;
+        if (feature.geometry.type === "LineString") {
+            measurement = length(feature);
+        } else if (feature.geometry.type === "Polygon") {
+            measurement = area(feature);
+        } else return info as DrawingPickInfo;
+        return {
+            ...info,
+            measurement: measurement,
+        } as DrawingPickInfo;
+    }
+
     // Callback for various editing events. Most events will update this component
     // through patches sent to the map parent. See patchLayerPropsin layerTools.ts.
     _onEdit(editAction: EditAction<FeatureCollection>): void {
         switch (editAction.editType) {
             case "addFeature":
-                patchLayerProps<FeatureCollection>(this, {
-                    ...this.props,
+                this.setState({
                     data: editAction.updatedData,
                     selectedFeatureIndexes:
                         editAction.editContext.featureIndexes,
-                } as DrawingLayerProps<FeatureCollection>);
+                });
+                (this.context as DeckGLLayerContext).userData.setEditedData({
+                    data: editAction.updatedData,
+                    selectedFeatureIndexes:
+                        editAction.editContext.featureIndexes,
+                });
                 break;
             case "removeFeature":
-                patchLayerProps<FeatureCollection>(this, {
-                    ...this.props,
+                this.setState({
                     data: editAction.updatedData,
                     selectedFeatureIndexes: [],
-                } as DrawingLayerProps<FeatureCollection>);
+                });
+                (this.context as DeckGLLayerContext).userData.setEditedData({
+                    data: editAction.updatedData,
+                    selectedFeatureIndexes: [] as number[],
+                });
                 break;
             case "removePosition":
             case "finishMovePosition":
-                patchLayerProps<FeatureCollection>(this, {
-                    ...this.props,
+                this.setState({
                     data: editAction.updatedData,
-                } as DrawingLayerProps<FeatureCollection>);
+                });
+                (this.context as DeckGLLayerContext).userData.setEditedData({
+                    data: editAction.updatedData,
+                });
                 break;
             case "movePosition":
-                // Don't use patchLayerProps to avoid an expensive roundtrip,
+            case "undoDrawing":
+                // Don't use setEditedData to avoid an expensive roundtrip,
                 // since this is done on every mouse move when editing.
                 this.setState({ data: editAction.updatedData });
                 break;
@@ -144,8 +198,8 @@ export default class DrawingLayer extends CompositeLayer<
     // Return the line color based on the selection status.
     // The same can be done for other features (polygons, points etc).
     _getLineColor(feature: Feature): RGBAColor {
-        const is_feature_selected = this.props.selectedFeatureIndexes.some(
-            (i) => this.state.data.features[i] === feature
+        const is_feature_selected = this.state.selectedFeatureIndexes.some(
+            (i: number) => this.state.data.features[i] === feature
         );
         if (is_feature_selected) {
             return SELECTED_LINE_COLOR;
@@ -166,7 +220,7 @@ export default class DrawingLayer extends CompositeLayer<
                     modeConfig: {
                         viewport: this.context.viewport,
                     },
-                    selectedFeatureIndexes: this.props.selectedFeatureIndexes,
+                    selectedFeatureIndexes: this.state.selectedFeatureIndexes,
                     coordinateSystem: COORDINATE_SYSTEM.CARTESIAN,
                     onEdit: (editAction: EditAction<FeatureCollection>) =>
                         this._onEdit(editAction),
@@ -184,4 +238,6 @@ export default class DrawingLayer extends CompositeLayer<
 }
 
 DrawingLayer.layerName = "DrawingLayer";
-DrawingLayer.defaultProps = defaultProps;
+DrawingLayer.defaultProps = layersDefaultProps[
+    "DrawingLayer"
+] as DrawingLayerProps<FeatureCollection>;
