@@ -1,5 +1,6 @@
 import { CompositeLayer } from "@deck.gl/core";
 import { ExtendedLayerProps, isDrawingEnabled } from "../utils/layerTools";
+import { Layer } from "@deck.gl/core";
 import { GeoJsonLayer, PathLayer, TextLayer } from "@deck.gl/layers";
 import { RGBAColor } from "@deck.gl/core/utils/color";
 import { PathStyleExtension } from "@deck.gl/extensions";
@@ -7,8 +8,9 @@ import { subtract, distance, dot } from "mathjs";
 import {
     rgbValues,
     colorTablesArray,
-    colorsArray,
+    getColors,
 } from "@emerson-eps/color-tables/";
+
 import {
     Feature,
     GeometryCollection,
@@ -30,27 +32,43 @@ import { Position2D } from "@deck.gl/core/utils/positions";
 import { layersDefaultProps } from "../layersDefaultProps";
 import { UpdateStateInfo } from "@deck.gl/core/lib/layer";
 import { DeckGLLayerContext } from "../../components/Map";
+import {
+    ContinuousLegendDataType,
+    DiscreteLegendDataType,
+} from "../../components/ColorLegend";
+import { getLayersById } from "../../layers/utils/layerTools";
+
+type StyleAccessorFunction = (
+    object: Feature,
+    objectInfo?: Record<string, unknown>
+) => LineStyle;
 
 type NumberPair = [number, number];
-type DashAccessorFunction = (
-    object: Record<string, unknown>,
-    objectInfo: Record<string, unknown>
-) => NumberPair;
-type DashAccessor = boolean | NumberPair | DashAccessorFunction | undefined;
+type DashAccessor = boolean | NumberPair | StyleAccessorFunction | undefined;
+type ColorAccessor = RGBAColor | StyleAccessorFunction | undefined;
+type WidthAccessor = number | StyleAccessor | undefined;
+type LineStyle = NumberPair | RGBAColor | number;
+
+type StyleAccessor = {
+    color?: ColorAccessor;
+    dash?: DashAccessor;
+    width?: WidthAccessor;
+};
 
 export interface WellsLayerProps<D> extends ExtendedLayerProps<D> {
     pointRadiusScale: number;
     lineWidthScale: number;
     outline: boolean;
     selectedWell: string;
-    logData: string | LogCurveDataType;
+    logData: string | LogCurveDataType[];
     logName: string;
     logColor: string;
     logrunName: string;
     logRadius: number;
     logCurves: boolean;
     refine: boolean;
-    dashed?: DashAccessor;
+    colorMappingFunction: (x: number) => [number, number, number];
+    lineStyle: StyleAccessor;
     wellNameVisible: boolean;
     wellNameAtTop: boolean;
     wellNameSize: number;
@@ -84,31 +102,79 @@ function multiply(pair: [number, number], factor: number): [number, number] {
     return [pair[0] * factor, pair[1] * factor];
 }
 
-const DEFAULT_DASH = [5, 5];
+const DEFAULT_LINE_WIDTH = 5;
+const DEFAULT_DASH = [5, 5] as NumberPair;
 
-function getDashFactor(accessor: DashAccessor, factor: number) {
+function getDashFactor(
+    accessor: DashAccessor,
+    width_accessor?: number | ((object: Feature) => number),
+    offset = 0
+) {
+    return (
+        object: Feature,
+        objectInfo: Record<string, unknown>
+    ): NumberPair => {
+        let width = DEFAULT_LINE_WIDTH;
+        if (typeof width_accessor == "function") {
+            width = (width_accessor as StyleAccessorFunction)(object) as number;
+        } else if (width_accessor as number) {
+            width = width_accessor as number;
+        }
+        const factor = width / (width + offset);
+
+        let dash: NumberPair = [0, 0];
+        if (typeof accessor == "function") {
+            dash = (accessor as StyleAccessorFunction)(
+                object,
+                objectInfo
+            ) as NumberPair;
+        } else if (accessor as NumberPair) dash = accessor as NumberPair;
+        else if (accessor) dash = DEFAULT_DASH;
+        if (dash.length == 2) {
+            return multiply(dash, factor);
+        } else {
+            return multiply(DEFAULT_DASH, factor);
+        }
+    };
+}
+
+function getLineColor(accessor: ColorAccessor) {
+    if (accessor as RGBAColor) {
+        return accessor as RGBAColor;
+    }
+
+    return (
+        object: Feature,
+        objectInfo?: Record<string, unknown>
+    ): RGBAColor => {
+        if (typeof accessor === "function") {
+            const color = (accessor as StyleAccessorFunction)(
+                object,
+                objectInfo
+            ) as RGBAColor;
+            if (color) {
+                return color;
+            }
+        }
+        return object.properties?.["color"] as RGBAColor;
+    };
+}
+
+function getLineWidth(
+    accessor: WidthAccessor,
+    offset = 0
+): number | ((object: Feature) => number) {
     if (typeof accessor == "function") {
-        // eslint-disable-next-line @typescript-eslint/no-explicit-any
-        return (
-            object: Record<string, unknown>,
-            objectInfo: Record<string, unknown>
-        ): NumberPair => {
-            return multiply(
-                (accessor as DashAccessorFunction)(
-                    object,
-                    objectInfo
-                ) as NumberPair,
-                factor
+        return (object: Feature): number => {
+            return (
+                ((accessor as StyleAccessorFunction)(object) as number) + offset
             );
         };
     }
 
-    let value = DEFAULT_DASH as NumberPair;
-    if ((accessor as NumberPair)?.length == 2) {
-        value = accessor as NumberPair;
-    }
+    if (accessor as number) return (accessor as number) + offset;
 
-    return multiply(value, factor);
+    return DEFAULT_LINE_WIDTH + offset;
 }
 
 export default class WellsLayer extends CompositeLayer<
@@ -123,10 +189,25 @@ export default class WellsLayer extends CompositeLayer<
             (this.context as DeckGLLayerContext).userData.setEditedData({
                 selectedWell: (info.object as Feature).properties?.["name"],
             });
+            /*this.props.selectedWell = (info.object as Feature).properties?.[
+                "name"
+            ];*/
             return true;
         }
     }
 
+    setSelection(
+        well: string | undefined,
+        _selection?: [number | undefined, number | undefined]
+    ): void {
+        if (this.internalState) {
+            this.setState({
+                well: well,
+                selection: _selection,
+            });
+        }
+    }
+
     shouldUpdateState({
         changeFlags,
     }: UpdateStateInfo<
@@ -139,6 +220,47 @@ export default class WellsLayer extends CompositeLayer<
         );
     }
 
+    getLegendData(
+        value: LogCurveDataType[]
+    ): ContinuousLegendDataType | DiscreteLegendDataType | null {
+        return getLegendData(
+            value,
+            "",
+            this.props.logName,
+            this.props.logColor
+        );
+    }
+
+    setLegend(value: LogCurveDataType[]): void {
+        this.setState({
+            legend: this.getLegendData(value),
+        });
+    }
+
+    getLogLayer(): Layer<unknown> {
+        return getLayersById(
+            this.internalState?.subLayers,
+            "wells-layer-log_curve"
+        )?.[0];
+    }
+
+    getSelectionLayer(): Layer<unknown> {
+        return getLayersById(
+            this.internalState?.subLayers,
+            "wells-layer-selection"
+        )?.[0];
+    }
+
+    getLogCurveData(): LogCurveDataType[] | undefined {
+        const log_layer = this.getLogLayer();
+        return log_layer?.internalState?.asyncProps?.data?.resolvedValue;
+    }
+
+    setupLegend(): void {
+        const data = this.getLogCurveData();
+        if (data) this.setLegend(data);
+    }
+
     renderLayers(): (
         | GeoJsonLayer<Feature>
         | PathLayer<LogCurveDataType>
@@ -154,9 +276,11 @@ export default class WellsLayer extends CompositeLayer<
             : (this.props.data as FeatureCollection);
 
         const is3d = this.context.viewport.constructor.name === "OrbitViewport";
-        const positionFormat = is3d ? "XYZ" : "XY";
+        const isOrthographic =
+            this.context.viewport.constructor.name === "OrthographicViewport";
+        const positionFormat = isOrthographic ? "XY" : "XYZ";
 
-        const isDashed = !!this.props.dashed;
+        const isDashed = !!this.props.lineStyle?.dash;
 
         const extensions = [
             new PathStyleExtension({
@@ -177,15 +301,14 @@ export default class WellsLayer extends CompositeLayer<
                 visible: this.props.outline,
                 pointRadiusScale: this.props.pointRadiusScale,
                 lineWidthScale: this.props.lineWidthScale,
+                getLineWidth: getLineWidth(this.props.lineStyle?.width),
                 extensions: extensions,
-                getDashArray: getDashFactor(this.props.dashed, 1),
+                getDashArray: getDashFactor(this.props.lineStyle?.dash),
+                lineBillboard: true,
+                pointBillboard: true,
             })
         );
 
-        const lineWidthFactor =
-            this.props.lineWidthScale / (this.props.lineWidthScale - 1);
-
-        const getColor = (d: Feature): RGBAColor => d?.properties?.["color"];
         const colors = new GeoJsonLayer<Feature>(
             this.getSubLayerProps<Feature>({
                 id: "colors",
@@ -196,11 +319,18 @@ export default class WellsLayer extends CompositeLayer<
                 pointRadiusUnits: "pixels",
                 lineWidthUnits: "pixels",
                 pointRadiusScale: this.props.pointRadiusScale - 1,
-                lineWidthScale: this.props.lineWidthScale - 1,
-                getFillColor: getColor,
-                getLineColor: getColor,
+                lineWidthScale: this.props.lineWidthScale,
+                getLineWidth: getLineWidth(this.props.lineStyle?.width, -1),
+                getFillColor: (d: Feature) => d.properties?.["color"],
+                getLineColor: getLineColor(this.props.lineStyle?.color),
                 extensions: extensions,
-                getDashArray: getDashFactor(this.props.dashed, lineWidthFactor),
+                getDashArray: getDashFactor(
+                    this.props.lineStyle?.dash,
+                    getLineWidth(this.props.lineStyle?.width),
+                    -1
+                ),
+                lineBillboard: true,
+                pointBillboard: true,
             })
         );
 
@@ -218,9 +348,10 @@ export default class WellsLayer extends CompositeLayer<
                 pointRadiusUnits: "pixels",
                 lineWidthUnits: "pixels",
                 pointRadiusScale: this.props.pointRadiusScale + 2,
-                lineWidthScale: this.props.lineWidthScale + 2,
-                getFillColor: getColor,
-                getLineColor: getColor,
+                lineWidthScale: this.props.lineWidthScale,
+                getLineWidth: getLineWidth(this.props.lineStyle?.width, 2),
+                getFillColor: (d: Feature) => d.properties?.["color"],
+                getLineColor: getLineColor(this.props.lineStyle?.color),
             })
         );
 
@@ -235,7 +366,12 @@ export default class WellsLayer extends CompositeLayer<
                 miterLimit: 100,
                 visible: this.props.logCurves,
                 getPath: (d: LogCurveDataType): Position[] =>
-                    getLogPath(data.features, d, this.props.logrunName),
+                    getLogPath(
+                        data.features,
+                        d,
+                        this.props.logrunName,
+                        this.props.lineStyle?.color
+                    ),
                 getColor: (d: LogCurveDataType): RGBAColor[] =>
                     getLogColor(
                         d,
@@ -243,7 +379,8 @@ export default class WellsLayer extends CompositeLayer<
                         this.props.logName,
                         this.props.logColor,
                         (this.context as DeckGLLayerContext).userData
-                            .colorTables
+                            .colorTables,
+                        this.props.colorMappingFunction
                     ),
                 getWidth: (d: LogCurveDataType): number | number[] =>
                     this.props.logRadius ||
@@ -252,10 +389,9 @@ export default class WellsLayer extends CompositeLayer<
                     getColor: [
                         this.props.logrunName,
                         this.props.logName,
+                        this.props.logColor,
                         (this.context as DeckGLLayerContext).userData
                             .colorTables,
-                        this.props.logName,
-                        this.props.logColor,
                     ],
                     getWidth: [
                         this.props.logrunName,
@@ -265,13 +401,61 @@ export default class WellsLayer extends CompositeLayer<
                     getPath: [positionFormat],
                 },
                 onDataLoad: (value: LogCurveDataType[]) => {
-                    this.setState({
-                        legend: getLegendData(
-                            value,
-                            this.props.logName,
-                            this.props.logColor
-                        ),
-                    });
+                    this.setLegend(value);
+                },
+            })
+        );
+
+        const selection_layer = new PathLayer<LogCurveDataType>(
+            this.getSubLayerProps<LogCurveDataType>({
+                id: "selection",
+                data: this.props.logData,
+                positionFormat,
+                pickable: false,
+                widthScale: 10,
+                widthMinPixels: 1,
+                miterLimit: 100,
+                visible: this.props.logCurves,
+                getPath: (d: LogCurveDataType): Position[] =>
+                    getLogPath1(
+                        data.features,
+                        d,
+                        this.state.well,
+                        this.state.selection,
+                        this.props.logrunName,
+                        this.props.lineStyle?.color
+                    ),
+                getColor: (d: LogCurveDataType): RGBAColor[] =>
+                    getLogColor1(
+                        data.features,
+                        d,
+                        this.state.well,
+                        this.state.selection,
+                        this.props.logrunName
+                    ),
+                getWidth: (d: LogCurveDataType): number | number[] =>
+                    this.props.logRadius * 1.5 ||
+                    getLogWidth(d, this.props.logrunName, this.props.logName),
+                updateTriggers: {
+                    getColor: [
+                        this.props.logrunName,
+                        this.state.well,
+                        this.state.selection,
+                    ],
+                    getWidth: [
+                        this.props.logrunName,
+                        this.props.logName,
+                        this.props.logRadius,
+                    ],
+                    getPath: [
+                        positionFormat,
+                        this.props.logrunName,
+                        this.state.well,
+                        this.state.selection,
+                    ],
+                },
+                onDataLoad: (value: LogCurveDataType[]) => {
+                    this.setLegend(value);
                 },
             })
         );
@@ -283,7 +467,12 @@ export default class WellsLayer extends CompositeLayer<
                 data: data.features,
                 visible: this.props.wellNameVisible,
                 getPosition: (d: Feature) =>
-                    getAnnotationPosition(d, this.props.wellNameAtTop, is3d),
+                    getAnnotationPosition(
+                        d,
+                        this.props.wellNameAtTop,
+                        is3d,
+                        this.props.lineStyle?.color
+                    ),
                 getText: (d: Feature) => d.properties?.["name"],
                 getColor: this.props.wellNameColor,
                 getAnchor: "start",
@@ -292,7 +481,7 @@ export default class WellsLayer extends CompositeLayer<
             })
         );
 
-        return [outline, log_layer, colors, highlight, names];
+        return [outline, log_layer, colors, highlight, selection_layer, names];
     }
 
     // For now, use `any` for the picking types because this function should
@@ -301,8 +490,34 @@ export default class WellsLayer extends CompositeLayer<
     getPickingInfo({ info }: { info: any }): any {
         if (!info.object) return info;
 
-        const md_property = getMdProperty(info.coordinate, info.object);
-        const tvd_property = getTvdProperty(info.coordinate, info.object);
+        let md_property = getMdProperty(
+            info.coordinate,
+            info.object,
+            this.props.lineStyle?.color
+        );
+        if (!md_property) {
+            md_property = getLogProperty(
+                info.coordinate,
+                (this.props.data as FeatureCollection).features,
+                info.object,
+                this.props.logrunName,
+                "MD"
+            );
+        }
+        let tvd_property = getTvdProperty(
+            info.coordinate,
+            info.object,
+            this.props.lineStyle?.color
+        );
+        if (!tvd_property) {
+            tvd_property = getLogProperty(
+                info.coordinate,
+                (this.props.data as FeatureCollection).features,
+                info.object,
+                this.props.logrunName,
+                "TVD"
+            );
+        }
         const log_property = getLogProperty(
             info.coordinate,
             (this.props.data as FeatureCollection).features,
@@ -349,7 +564,7 @@ function getColumn<D>(data: D[][], col: number): D[] {
 function getLogMd(d: LogCurveDataType, logrun_name: string): number[] {
     if (!isSelectedLogRun(d, logrun_name)) return [];
 
-    const names_md = ["DEPTH", "DEPT", "MD", "TDEP"]; // aliases for MD
+    const names_md = ["DEPTH", "DEPT", "MD", "TDEP", "MD_RKB"]; // aliases for MD
     const log_id = getLogIndexByNames(d, names_md);
     return log_id >= 0 ? getColumn(d.data, log_id) : [];
 }
@@ -388,7 +603,8 @@ function isSelectedLogRun(d: LogCurveDataType, logrun_name: string): boolean {
 function getAnnotationPosition(
     well_data: Feature,
     name_at_top: boolean,
-    view_is_3d: boolean
+    view_is_3d: boolean,
+    color_accessor: ColorAccessor
 ): Position | null {
     if (name_at_top) {
         let top;
@@ -397,7 +613,7 @@ function getAnnotationPosition(
         const well_head = getWellHeadPosition(well_data);
         if (well_data) top = well_head;
         else {
-            const trajectory = getTrajectory(well_data);
+            const trajectory = getTrajectory(well_data, color_accessor);
             top = trajectory?.at(0);
         }
 
@@ -406,7 +622,7 @@ function getAnnotationPosition(
     } else {
         let bot;
         // if trajectory is not present, return top position from Point geometry
-        const trajectory = getTrajectory(well_data);
+        const trajectory = getTrajectory(well_data, color_accessor);
         if (trajectory) bot = trajectory?.at(-1);
         else bot = getWellHeadPosition(well_data);
 
@@ -426,13 +642,13 @@ function getWellObjectByName(
     );
 }
 
-function getPointGeometry(well_object: Feature): Point | undefined {
+function getPointGeometry(well_object: Feature): Point {
     return (well_object.geometry as GeometryCollection)?.geometries.find(
         (item) => item.type == "Point"
     ) as Point;
 }
 
-function getLineStringGeometry(well_object: Feature): LineString | undefined {
+function getLineStringGeometry(well_object: Feature): LineString {
     return (well_object.geometry as GeometryCollection)?.geometries.find(
         (item) => item.type == "LineString"
     ) as LineString;
@@ -443,9 +659,29 @@ function getWellHeadPosition(well_object: Feature): Position | undefined {
     return getPointGeometry(well_object)?.coordinates;
 }
 
-// Return Trajectory data from LineString Geometry
-function getTrajectory(well_object: Feature): Position[] | undefined {
-    return getLineStringGeometry(well_object)?.coordinates;
+// return trajectory visibility based on alpha of trajectory color
+function isTrajectoryVisible(
+    well_object: Feature,
+    color_accessor: ColorAccessor
+): boolean {
+    let alpha;
+    const accessor = getLineColor(color_accessor);
+    if (typeof accessor === "function") {
+        alpha = accessor(well_object)?.[3];
+    } else {
+        alpha = (accessor as RGBAColor)?.[3];
+    }
+    return alpha !== 0;
+}
+
+// Return Trajectory data from LineString Geometry if it's visible (checking trajectory visiblity based on line color)
+function getTrajectory(
+    well_object: Feature,
+    color_accessor: ColorAccessor
+): Position[] | undefined {
+    if (isTrajectoryVisible(well_object, color_accessor))
+        return getLineStringGeometry(well_object)?.coordinates;
+    else return undefined;
 }
 
 function getWellMds(well_object: Feature): number[] {
@@ -457,15 +693,26 @@ function getNeighboringMdIndices(mds: number[], md: number): number[] {
     return idx === 0 ? [idx, idx + 1] : [idx - 1, idx];
 }
 
+function getPositionByMD(well_xyz: Position[], well_mds: number[], md: number) {
+    const [l_idx, h_idx] = getNeighboringMdIndices(well_mds, md);
+    const md_low = well_mds[l_idx];
+    const md_normalized = (md - md_low) / (well_mds[h_idx] - md_low);
+    return interpolateNumberArray(
+        well_xyz[l_idx],
+        well_xyz[h_idx]
+    )(md_normalized);
+}
+
 function getLogPath(
     wells_data: Feature[],
     d: LogCurveDataType,
-    logrun_name: string
+    logrun_name: string,
+    trajectory_line_color?: ColorAccessor
 ): Position[] {
     const well_object = getWellObjectByName(wells_data, d.header.well);
     if (!well_object) return [];
 
-    const well_xyz = getTrajectory(well_object);
+    const well_xyz = getTrajectory(well_object, trajectory_line_color);
     const well_mds = getWellMds(well_object);
 
     if (
@@ -479,15 +726,10 @@ function getLogPath(
     const log_xyz: Position[] = [];
     const log_mds = getLogMd(d, logrun_name);
     log_mds.forEach((md) => {
-        const [l_idx, h_idx] = getNeighboringMdIndices(well_mds, md);
-        const md_normalized =
-            (md - well_mds[l_idx]) / (well_mds[h_idx] - well_mds[l_idx]);
-        const xyz = interpolateNumberArray(
-            well_xyz[l_idx],
-            well_xyz[h_idx]
-        )(md_normalized);
+        const xyz = getPositionByMD(well_xyz, well_mds, md);
         log_xyz.push(xyz);
     });
+
     return log_xyz;
 }
 
@@ -512,23 +754,25 @@ function getLogColor(
     logrun_name: string,
     log_name: string,
     logColor: string,
-    colorTables: colorTablesArray
+    colorTables: colorTablesArray,
+    // eslint-disable-next-line
+    colorMappingFunction: any
 ): RGBAColor[] {
     const log_data = getLogValues(d, logrun_name, log_name);
     const log_info = getLogInfo(d, logrun_name, log_name);
-
     if (log_data.length == 0 || log_info == undefined) return [];
     const log_color: RGBAColor[] = [];
+
     if (log_info.description == "continuous") {
         const min = Math.min(...log_data);
         const max = Math.max(...log_data);
         const max_delta = max - min;
+
         log_data.forEach((value) => {
-            const rgb = rgbValues(
-                (value - min) / max_delta,
-                logColor,
-                colorTables
-            );
+            const rgb = colorMappingFunction
+                ? colorMappingFunction((value - min) / max_delta)
+                : rgbValues((value - min) / max_delta, logColor, colorTables);
+
             if (rgb) {
                 if (Array.isArray(rgb)) {
                     log_color.push([rgb[0], rgb[1], rgb[2]]);
@@ -540,26 +784,46 @@ function getLogColor(
             }
         });
     } else {
-        const arrayOfColors: [number, number, number, number][] = colorsArray(
-            logColor,
-            colorTables
-        );
-
+        // well log data set for ex : H1: Array(2)0: (4) [255, 26, 202, 255] 1: 13
         const log_attributes = getDiscreteLogMetadata(d, log_name)?.objects;
+        const logLength = Object.keys(log_attributes).length;
+
         // eslint-disable-next-line
         const attributesObject: { [key: string]: any } = {};
+        const categorial = true;
+
         Object.keys(log_attributes).forEach((key) => {
-            // get the code from log_attributes
-            const code = log_attributes[key][1];
-            // compare the code and first value from colorsArray(colortable)
-            const colorArrays = arrayOfColors.find((value: number[]) => {
-                return value[0] == code;
-            });
-            if (colorArrays)
-                attributesObject[key] = [
-                    [colorArrays[1], colorArrays[2], colorArrays[3]],
-                    code,
-                ];
+            // get the point from log_attributes
+            const point = log_attributes[key][1];
+            const categorialMin = 0;
+            const categorialMax = logLength - 1;
+
+            let rgb;
+            if (colorMappingFunction) {
+                rgb = colorMappingFunction(
+                    point,
+                    categorial,
+                    categorialMin,
+                    categorialMax
+                );
+            } else {
+                // if colormap function is not defined
+                const arrayOfColors: [number, number, number, number][] =
+                    getColors(logColor, colorTables, point);
+                if (!arrayOfColors.length)
+                    console.error(
+                        "Empty or missed '" + logColor + "' color table"
+                    );
+                rgb = arrayOfColors;
+            }
+
+            if (rgb) {
+                if (Array.isArray(rgb)) {
+                    attributesObject[key] = [[rgb[1], rgb[2], rgb[3]], point];
+                } else {
+                    attributesObject[key] = [[rgb.r, rgb.g, rgb.b], point];
+                }
+            }
         });
         log_data.forEach((log_value) => {
             const dl_attrs = Object.entries(attributesObject).find(
@@ -573,6 +837,130 @@ function getLogColor(
     return log_color;
 }
 
+function getLogPath1(
+    wells_data: Feature[],
+    d: LogCurveDataType,
+    selectedWell: string | undefined,
+    selection: [number | undefined, number | undefined] | undefined,
+    logrun_name: string,
+    trajectory_line_color?: ColorAccessor
+): Position[] {
+    if (!selection || selectedWell !== d.header.well) return [];
+    const well_object = getWellObjectByName(wells_data, d.header.well);
+    if (!well_object) return [];
+
+    const well_xyz = getTrajectory(well_object, trajectory_line_color);
+    const well_mds = getWellMds(well_object);
+
+    if (
+        well_xyz == undefined ||
+        well_mds == undefined ||
+        well_xyz.length == 0 ||
+        well_mds.length == 0
+    )
+        return [];
+
+    const log_mds = getLogMd(d, logrun_name);
+    if (!log_mds) return [];
+
+    const log_xyz: Position[] = [];
+
+    let md0 = selection[0] as number;
+    if (md0 !== undefined) {
+        let md1 = selection[1];
+        if (md1 == md0) md1 = undefined;
+        const mdFirst = well_mds[0];
+        const mdLast = well_mds[well_mds.length - 1];
+
+        if (md1 !== undefined) {
+            if (md0 > md1) {
+                const tmp: number = md0;
+                md0 = md1;
+                md1 = tmp;
+            }
+        }
+
+        const delta = 2;
+        if (md0 - delta > mdFirst) {
+            let xyz = getPositionByMD(well_xyz, well_mds, md0 - delta);
+            log_xyz.push(xyz);
+            xyz = getPositionByMD(well_xyz, well_mds, md0);
+            log_xyz.push(xyz);
+        }
+        if (md1 !== undefined) {
+            const _md1 = md1 as number;
+            let index = 0;
+            well_mds.forEach((md) => {
+                if (md0 <= md && md <= _md1) {
+                    const xyz = well_xyz[index];
+                    log_xyz.push(xyz);
+                }
+                index++;
+            });
+            if (_md1 + delta < mdLast) {
+                let xyz = getPositionByMD(well_xyz, well_mds, _md1);
+                log_xyz.push(xyz);
+                xyz = getPositionByMD(well_xyz, well_mds, _md1 + delta);
+                log_xyz.push(xyz);
+            }
+        }
+    }
+    return log_xyz;
+}
+
+function getLogColor1(
+    wells_data: Feature[],
+    d: LogCurveDataType,
+    selectedWell: string | undefined,
+    selection: [number | undefined, number | undefined] | undefined,
+    logrun_name: string
+): RGBAColor[] {
+    if (!selection || selectedWell !== d.header.well) return [];
+    const well_object = getWellObjectByName(wells_data, d.header.well);
+    if (!well_object) return [];
+
+    const well_mds = getWellMds(well_object);
+
+    const log_mds = getLogMd(d, logrun_name);
+    if (!log_mds || log_mds.length === 0) return [];
+
+    const log_color: RGBAColor[] = [];
+
+    let md0 = selection[0] as number;
+    if (md0 !== undefined) {
+        const mdFirst = well_mds[0];
+        const mdLast = well_mds[well_mds.length - 1];
+        let md1 = selection[1];
+        if (md1 == md0) md1 = undefined;
+        let swap = false;
+        if (md1 !== undefined) {
+            if (md0 > md1) {
+                const tmp: number = md0;
+                md0 = md1;
+                md1 = tmp;
+                swap = true;
+            }
+        }
+        const delta = 2;
+        if (md0 - delta > mdFirst)
+            log_color.push(swap ? [0, 255, 0, 128] : [255, 0, 0, 128]);
+
+        if (md1 !== undefined) {
+            const _md1 = md1 as number;
+            log_color.push([128, 128, 128, 128]);
+            well_mds.forEach((md) => {
+                if (md0 <= md && md <= _md1) {
+                    log_color.push([128, 128, 128, 128]);
+                }
+            });
+
+            if (_md1 + delta < mdLast)
+                log_color.push(swap ? [255, 0, 0, 128] : [0, 255, 0, 128]);
+        }
+    }
+    return log_color;
+}
+
 function getLogWidth(
     d: LogCurveDataType,
     logrun_name: string,
@@ -641,11 +1029,15 @@ function interpolateDataOnTrajectory(
     return data0 * (1.0 - scalar_projection) + data1 * scalar_projection;
 }
 
-function getMd(coord: Position, feature: Feature): number | null {
+function getMd(
+    coord: Position,
+    feature: Feature,
+    accessor: ColorAccessor
+): number | null {
     if (!feature.properties?.["md"]?.[0] || !feature.geometry) return null;
 
     const measured_depths = feature.properties["md"][0] as number[];
-    const trajectory3D = getTrajectory(feature);
+    const trajectory3D = getTrajectory(feature, accessor);
 
     if (trajectory3D == undefined) return null;
 
@@ -666,9 +1058,10 @@ function getMd(coord: Position, feature: Feature): number | null {
 
 function getMdProperty(
     coord: Position,
-    feature: Feature
+    feature: Feature,
+    accessor: ColorAccessor
 ): PropertyDataType | null {
-    const md = getMd(coord, feature);
+    const md = getMd(coord, feature, accessor);
     if (md != null) {
         const prop_name = "MD " + feature.properties?.["name"];
         return createPropertyData(prop_name, md, feature.properties?.["color"]);
@@ -676,8 +1069,12 @@ function getMdProperty(
     return null;
 }
 
-function getTvd(coord: Position, feature: Feature): number | null {
-    const trajectory3D = getTrajectory(feature);
+function getTvd(
+    coord: Position,
+    feature: Feature,
+    accessor: ColorAccessor
+): number | null {
+    const trajectory3D = getTrajectory(feature, accessor);
 
     // if trajectory is not found or if it has a data single point then get tvd from well head
     if (trajectory3D == undefined || trajectory3D?.length <= 1) {
@@ -704,9 +1101,10 @@ function getTvd(coord: Position, feature: Feature): number | null {
 
 function getTvdProperty(
     coord: Position,
-    feature: Feature
+    feature: Feature,
+    accessor: ColorAccessor
 ): PropertyDataType | null {
-    const tvd = getTvd(coord, feature);
+    const tvd = getTvd(coord, feature, accessor);
     if (tvd != null) {
         const prop_name = "TVD " + feature.properties?.["name"];
         return createPropertyData(
@@ -792,23 +1190,27 @@ function getLogProperty(
 // Return data required to build welllayer legend
 function getLegendData(
     logs: LogCurveDataType[],
+    wellName: string,
     logName: string,
     logColor: string
-) {
-    const logInfo = getLogInfo(logs[0], logs[0].header.name, logName);
+): ContinuousLegendDataType | DiscreteLegendDataType | null {
+    if (!logs) return null;
+    const log = wellName
+        ? logs.find((log) => log.header.well == wellName)
+        : logs[0];
+    const logInfo = !log
+        ? undefined
+        : getLogInfo(log, log.header.name, logName);
     const title = "Wells / " + logName;
-    const legendProps = [];
-    if (logInfo?.description == "discrete") {
-        const meta = logs[0]["metadata_discrete"];
+    if (log && logInfo?.description == "discrete") {
+        const meta = log["metadata_discrete"];
         const metadataDiscrete = meta[logName].objects;
-        legendProps.push({
+        return {
             title: title,
             colorName: logColor,
             discrete: true,
             metadata: metadataDiscrete,
-            valueRange: [],
-        });
-        return legendProps;
+        };
     } else {
         const minArray: number[] = [];
         const maxArray: number[] = [];
@@ -817,14 +1219,11 @@ function getLegendData(
             minArray.push(Math.min(...logValues));
             maxArray.push(Math.max(...logValues));
         });
-        legendProps.push({
+        return {
             title: title,
-            name: logName,
             colorName: logColor,
             discrete: false,
-            metadata: { objects: {} },
             valueRange: [Math.min(...minArray), Math.max(...maxArray)],
-        });
-        return legendProps;
+        };
     }
 }
