@@ -1,6 +1,9 @@
 import React from "react";
 import DeckGLMap from "../DeckGLMap";
 import exampleData from "../../../../demo/example-data/deckgl-map.json";
+import { makeStyles } from "@material-ui/styles";
+import { ColorLegend, createColorMapFunction } from "@emerson-eps/color-tables";
+const colorTables = require("@emerson-eps/color-tables/dist/component/color-tables.json");
 
 export default {
     component: DeckGLMap,
@@ -81,7 +84,7 @@ export default {
             description:
                 "Prop containing color table data." +
                 "See colorTables repo for reference:<br/>" +
-                "https://github.com/emerson-eps/color-tables/blob/main/react-app/src/component/color-tables.json",
+                "https://github.com/emerson-eps/color-tables/blob/main/react-app/dist/component/color-tables.json",
         },
 
         editedData: {
@@ -142,7 +145,7 @@ const customLayerWithPolylineData = {
             },
         ],
     },
-    lineWidthScale: 20,
+    getLineWidth: 20,
     lineWidthMinPixels: 2,
 };
 
@@ -167,7 +170,7 @@ const customLayerWithPolygonData = {
             ],
         },
     },
-    lineWidthScale: 20,
+    getLineWidth: 20,
     lineWidthMinPixels: 2,
     getLineColor: [0, 255, 255],
     getFillColor: [0, 255, 0],
@@ -275,12 +278,14 @@ const meshMapLayer = {
     mesh: "hugin_depth_25_m_normalized_margin.png",
     meshValueRange: [2782, 3513],
     propertyTexture: "kh_netmap_25_m_normalized_margin.png",
-    propertyValueRange: [2782, 3513],
-    rotDeg: 0,
-    contours: [0, 50.0],
-    isContoursDepth: false,
+    propertyValueRange: [-3071, 41048],
+    rotDeg: 0, // default rotate around bounds' upper left corner.
+    //rotPoint: [432205 + (439400 - 432205) / 2, 6475078 + (6481113 - 6475078) / 2],  // rotate around middle
+    contours: [0, 100.0],
+    isContoursDepth: true,
     colorMapName: "Physics",
 };
+
 export const KhMapMesh = MinimalTemplate.bind({});
 KhMapMesh.args = {
     id: "kh-mesh-map",
@@ -394,10 +399,47 @@ const axes = {
     id: "axes-layer",
     bounds: [432205, 6475078, -3500, 437720, 6481113, 0],
 };
+const north_arrow_layer = {
+    "@@type": "NorthArrow3DLayer",
+    id: "north-arrow-layer",
+};
+
 export const Axes = MinimalTemplate.bind({});
 Axes.args = {
     id: "axes",
-    layers: [meshMapLayer, axes],
+    layers: [axes, meshMapLayer, north_arrow_layer],
+    bounds: [432150, 6475800, 439400, 6481500],
+    views: {
+        layout: [1, 1],
+        viewports: [
+            {
+                id: "view_1",
+                show3D: false,
+                layerIds: [],
+            },
+        ],
+    },
+};
+
+// Example using "colorMapFunction" property.
+const layer = {
+    ...meshMapLayer,
+    isContoursDepth: true,
+    colorMapFunction: (x) => [255 - x * 100, 255 - x * 100, 255 * x], // If defined this function will override the colormap.
+};
+export const colorMapFunction = MinimalTemplate.bind({});
+colorMapFunction.args = {
+    id: "colorMapFunction",
+    layers: [
+        // map layer
+        layer,
+        // colormap layer
+        {
+            ...colormapLayer,
+            image: "https://raw.githubusercontent.com/equinor/webviz-subsurface-components/master/react/src/demo/example-data/propertyMap.png",
+            colorMapFunction: (x) => [255 - x * 100, 255 - x * 100, 255 * x], // If defined this function will override the colormap.
+        },
+    ],
     bounds: [432150, 6475800, 439400, 6481500],
     views: {
         layout: [1, 1],
@@ -421,6 +463,7 @@ GridLayer.args = {
             ...gridLayer,
             visible: true,
         },
+        axes,
     ],
     toolbar: {
         visible: false,
@@ -592,3 +635,156 @@ SelectableFeatureExample.args = {
         polygonUsingSelectableGeoJsonLayer,
     ],
 };
+
+// Map used inside a div container template
+const useStyles = makeStyles({
+    main: {
+        width: 500,
+        height: 500,
+        top: "50%",
+        left: "50%",
+        transform: "translate(-50%, -50%)",
+        border: "1px solid black",
+        background: "azure",
+        position: "absolute",
+    },
+});
+
+export const MapInContainer = (args) => {
+    const classes = useStyles();
+    return (
+        <div className={classes.main}>
+            <DeckGLMap {...args} />
+        </div>
+    );
+};
+
+MapInContainer.args = {
+    ...exampleData[0],
+};
+
+export const MultiColorMap = EditDataTemplate.bind({});
+MultiColorMap.args = {
+    ...exampleData[0],
+    legend: {
+        visible: true,
+    },
+    zoom: -5,
+    layers: [
+        exampleData[0].layers[0],
+        {
+            ...exampleData[0].layers[0],
+            colorMapRange: [3000, 3100],
+            valueRange: [3000, 3100],
+            id: "colormap-2-layer",
+        },
+    ],
+    views: {
+        layout: [1, 2],
+        showLabel: true,
+        viewports: [
+            {
+                id: "view_1",
+                name: "Colormap layer",
+                show3D: false,
+                layerIds: ["colormap-layer"],
+            },
+            {
+                id: "view_2",
+                name: "Colormap 2 layer",
+                show3D: false,
+                layerIds: ["colormap-2-layer"],
+            },
+        ],
+    },
+};
+
+// ColormapLayer with color selector component
+const defaultProps = {
+    id: "DeckGlMap",
+    resources: {
+        propertyMap:
+            "https://raw.githubusercontent.com/equinor/webviz-subsurface-components/master/react/src/demo/example-data/propertyMap.png",
+    },
+    bounds: [432150, 6475800, 439400, 6481500],
+};
+
+const layers = [
+    {
+        "@@type": "ColormapLayer",
+        image: "@@#resources.propertyMap",
+        rotDeg: 0,
+        bounds: [432205, 6475078, 437720, 6481113],
+        valueRange: [2782, 3513],
+        colorMapRange: [2782, 3513],
+    },
+];
+
+// prop for legend
+const min = 0;
+const max = 0.35;
+const dataObjectName = "Legend";
+const position = [16, 10];
+const horizontal = true;
+const colorName = "Physics";
+
+const mapDataTemplate = (args) => {
+    const [getColorName, setColorName] = React.useState();
+
+    const colorMapData = React.useCallback((data) => {
+        setColorName(data);
+    }, []);
+
+    const updatedLayerData = [
+        {
+            ...args.layers[0],
+            colorMapName: getColorName,
+            colorMapFunction: createColorMapFunction(
+                getColorName ? getColorName : colorName
+            ),
+        },
+    ];
+
+    return (
+        <div>
+            <div
+                style={{
+                    float: "right",
+                    zIndex: 999,
+                    opacity: 1,
+                    position: "relative",
+                }}
+            >
+                <ColorLegend {...args} getColorMapname={colorMapData} />
+            </div>
+            <DeckGLMap {...args} layers={updatedLayerData} />
+        </div>
+    );
+};
+
+export const ColorMapLayerColorSelector = mapDataTemplate.bind({});
+
+ColorMapLayerColorSelector.args = {
+    min,
+    max,
+    dataObjectName,
+    position,
+    horizontal,
+    colorName,
+    colorTables,
+    layers,
+    ...defaultProps,
+    legend: {
+        visible: false,
+    },
+};
+
+ColorMapLayerColorSelector.parameters = {
+    docs: {
+        description: {
+            story: "Clicking on legend opens(toggle) the color selector component and then click on the color scale to update the layer.",
+        },
+        inlineStories: false,
+        iframeHeight: 500,
+    },
+};
