@@ -14,7 +14,7 @@ import { WellLogController } from "./WellLogView";
 import Scroller from "./Scroller";
 
 interface Props {
-    welllog: WellLog;
+    welllog: WellLog | undefined;
     template: Template;
     colorTables: ColorTable[];
     horizontal?: boolean;
@@ -74,17 +74,6 @@ class WellLogViewWithScroller extends Component<Props> {
     }
 
     shouldComponentUpdate(nextProps: Props): boolean {
-        //console.log("is=", Object.is(this.props, nextProps));
-        /*
-            //compare (Object.keys(nextProps), Object.keys(this.props))
-            for (const p in nextProps) {
-                // eslint-disable-next-line
-                if ((nextProps as any)[p] !== (this.props as any)[p]) {
-                    console.log(p /*, nextProps[p], this.props[p]* /);
-                    return true;
-                }
-            }
-        return false;*/
         return !Object.is(this.props, nextProps);
     }
 
@@ -123,13 +112,11 @@ class WellLogViewWithScroller extends Component<Props> {
     // callback function from Scroller
     onScrollerScroll(x: number, y: number): void {
         const controller = this.controller;
-        if (controller) {
-            const fContent = this.props.horizontal ? x : y; // fraction
-            controller.scrollContentTo(fContent);
-
-            const posTrack = this.calcPosTrack(this.props.horizontal ? y : x);
-            controller.scrollTrackTo(posTrack);
-        }
+        if (!controller) return;
+        const fContent = this.props.horizontal ? x : y; // fraction
+        controller.scrollContentTo(fContent);
+        const posTrack = this.calcPosTrack(this.props.horizontal ? y : x);
+        controller.scrollTrackTo(posTrack);
     }
 
     calcPosTrack(f: number): number {
